"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ValidatorFieldError_1 = __importDefault(require("./Errors/ValidatorFieldError"));
const isemail_1 = __importDefault(require("isemail"));
const moment_1 = __importDefault(require("moment"));
const DefaultErrors_1 = __importDefault(require("./Errors/Dictionary/DefaultErrors"));
class ValidatorField {
    constructor() {
        this.required = false;
        this.default = null;
        this.message = null;
        this.errorsEnabled = true;
        this.queue = [];
        this.errors = Object.assign({}, DefaultErrors_1.default);
    }
    addQueue(fn) {
        this.queue.push(fn);
        return this;
    }
    /**
     * Заменяет стандартные сообщения об ошибках
     */
    setCustomErrors(errors = {}) {
        this.errors = Object.assign({}, this.errors, errors);
    }
    /**
     * Устанавливает дефолтное значение
     */
    setDefault(def) {
        this.default = def;
        return this;
    }
    /**
     * Помечает поле обязательным
     */
    setRequired() {
        this.required = true;
        return this;
    }
    /**
     * Устанавливает актуальное Сообщение об ошибке
     */
    setMessage(message) {
        return this.addQueue(item => {
            this.message = message;
        });
    }
    /**
     * Сбрасывает актуальное сообщение об ошибке
     */
    clearMessage() {
        return this.addQueue(item => {
            this.message = null;
        });
    }
    /**
     * NO ERRORS мод
     */
    errNo() {
        this.errorsEnabled = false;
        return this;
    }
    ////////////////////////////////////////////////////////////
    ////////////////// Validation methods //////////////////////
    ////////////////////////////////////////////////////////////
    /**
     * Значние является строкой
     */
    isString() {
        return this
            .try(this.errors.isString, () => {
            this.custom(item => String(item))
                .assert(item => typeof item === 'string');
        });
    }
    /**
     * Значение является числом
     */
    isNumeric() {
        return this
            .try(this.errors.isNumeric, () => {
            this.custom(item => +item)
                .assert(item => item >= -Infinity && item <= Infinity);
        });
    }
    /**
     * Значение является целым числом
     */
    isInt() {
        return this
            .try(this.errors.isInt, () => {
            return this.custom(item => parseInt(item))
                .assert(item => !isNaN(item));
        });
    }
    /**
     * Значение является булевым
     */
    isBoolean() {
        return this.custom(item => !!item);
    }
    /**
     * Проверка длинны строки
     */
    length(min, max) {
        return this
            .try(this.errors.length, () => {
            this.assert(item => item.length >= min && item.length <= max);
        });
    }
    /**
     * Проверка числа
     */
    range(min, max) {
        return this
            .try(this.errors.range, () => {
            this.assert(item => item >= min && item <= max);
        });
    }
    /**
     * Число больше или равно
     */
    min(min) {
        return this
            .try(this.errors.min, () => {
            this.assert(item => item >= min);
        });
    }
    /**
     * Число меньше или равно
     */
    max(max) {
        return this
            .try(this.errors.max, () => {
            this.assert(item => item <= max);
        });
    }
    /**
     * Число больше
     */
    after(min) {
        return this
            .try(this.errors.after, () => {
            this.assert(item => item > min);
        });
    }
    /**
     * Число меньше
     */
    before(max) {
        return this
            .try(this.errors.before, () => {
            this.assert(item => item < max);
        });
    }
    /**
     * Валидная карта
     */
    isCreditCard() {
        return this
            .try(this.errors.isCreditCard, () => {
            this.assert(item => {
                if (item.length < 16) {
                    return false;
                }
                let s = item.split('').reverse().join('');
                let sum = 0;
                for (let i = 0, j = s.length, val; i < j; i++) {
                    if ((i % 2) == 0) {
                        val = +s[i];
                    }
                    else {
                        val = +s[i] * 2;
                        if (val > 9) {
                            val -= 9;
                        }
                    }
                    sum += val;
                }
                return ((sum % 10) == 0);
            });
        });
    }
    /**
     * Преобразование в дату
     */
    isDate(format = 'YYYY-MM-DD') {
        return this
            .try(this.errors.isDate, () => {
            this.custom(item => (0, moment_1.default)(item, format, true))
                .assert(item => item.isValid())
                .custom(item => item.toDate());
        });
    }
    /**
     * Корректный E-mail
     */
    isEmail() {
        return this
            .try(this.errors.isEmail, () => {
            this.assert(item => isemail_1.default.validate(item));
        });
    }
    /**
     * Валидный json
     */
    isJSON(parse = true) {
        return this
            .try(this.errors.isJSON, () => {
            if (parse) {
                return this.custom(item => JSON.parse(item));
            }
            return this.assert(item => JSON.parse(item));
        });
    }
    /**
     * Нижний регистр
     */
    isLowerCase() {
        return this
            .try(this.errors.isLowerCase, () => {
            this.assert(item => item === item.toLowerCase());
        });
    }
    /**
     * Верхний регистр
     */
    isUpperCase() {
        return this
            .try(this.errors.isUpperCase, () => {
            this.assert(item => item === item.toUpperCase());
        });
    }
    /**
     * Обрезает пробельные символы строки
     */
    trim() {
        return this
            .try(this.errors.trim, () => {
            this.custom(item => item.trim());
        });
    }
    /**
     * Является массивом
     */
    isArray() {
        return this
            .try(this.errors.isArray, () => {
            this.assert(item => item instanceof Array);
        });
    }
    isObject() {
        return this
            .try(this.errors.isObject, () => {
            this.assert(item => (item instanceof Object));
        });
    }
    /**
     * Значение одно из
     */
    in(values) {
        return this
            .try(this.errors.in, () => {
            this.assert(item => values.indexOf(item) >= 0);
        });
    }
    /**
     * Значение НЕ одно из
     */
    notIn(values) {
        return this
            .try(this.errors.notIn, () => {
            this.assert(item => values.indexOf(item) < 0);
        });
    }
    /**
     * Тест регуляркой
     */
    regex(reg) {
        return this
            .try(this.errors.regex, () => {
            this.assert(item => reg.test(item));
        });
    }
    /**
     * Поиск по регулярке
     */
    regexSearch(reg) {
        return this
            .try(this.errors.regex, () => {
            this.custom(item => reg.exec(item))
                .assert(item => item !== null);
        });
    }
    /**
     * Удаление тегов
     */
    stripTags() {
        return this
            .try(this.errors.stripTags, () => {
            this.custom(item => item.replace(/<.+?(>|$)/g, ''));
        });
    }
    /**
     * Экранирует специальные символы HTML
     */
    encodeHtmlChars() {
        let replaces = {
            '&': '&amp;',
            '"': '&quot;',
            "'": '&#039;',
            "<": '&lt;',
            ">": '&gt;'
        };
        return this
            .try(this.errors.encodeHtmlChars, () => {
            this.custom(item => {
                if (typeof document !== 'undefined') {
                    let el = document.createElement('div');
                    el.textContent = item;
                    return el.innerHTML;
                }
                for (let symbol in replaces) {
                    item = item.replace(new RegExp(symbol, 'g'), replaces[symbol]);
                }
                return item;
            });
        });
    }
    /**
     * Декодирует URL-компонент
     */
    urlDecode() {
        return this
            .try(this.errors.urlDecode, () => {
            this.custom(item => encodeURIComponent(item));
        });
    }
    /**
     * Строка является ключем в объекте
     */
    inObjectKeys(obj) {
        return this
            .try(this.errors.inObjectKeys, () => {
            this.assert(item => (item in obj));
        });
    }
    ////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////
    try(message, fn) {
        return this.setMessage(message), fn(this), this.clearMessage();
    }
    assert(fn, message = null) {
        if (message) {
            return this.try(message, () => {
                this.assert(fn);
            });
        }
        return this.addQueue(item => {
            if (!fn(item)) {
                throw new ValidatorFieldError_1.default(this.message || 'Unknown validate error');
            }
        });
    }
    custom(fn) {
        return this.addQueue(fn);
    }
    validate(value) {
        if (value === undefined || value === null) {
            if (this.required && this.errorsEnabled) {
                throw new ValidatorFieldError_1.default(this.errors.required);
            }
            return this.default;
        }
        try {
            return this.queue.reduce((prev, fn) => {
                let next = fn(prev);
                if (next === undefined) {
                    return prev;
                }
                return next;
            }, value);
        }
        catch (e) {
            if (this.errorsEnabled) {
                if (e instanceof ValidatorFieldError_1.default) {
                    throw e;
                }
                throw new ValidatorFieldError_1.default(this.message || 'Unknown validate error');
            }
            return this.default;
        }
    }
}
exports.default = ValidatorField;
